/*  osdchatd - on-screen display messaging daemon
 *  Copyright (c) 2003 by Mike Smelov  <mike-at-argoncorp-dot-com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  See LICENSE for details.
 *
 */

 /*  osdchatd.h  */

#ifndef __OSDCHATD_H__
#define __OSDCHATD_H__

#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <pwd.h>
#include <stdlib.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <xosd.h>

#define PID_FILE		"/tmp/osdchatd-%d.pid"		/* pid file name based on uid */
#define LOG_FILE		".osdchat.log"

#define CONFIG_USER		".osdchatd"
#define CONFIG_SYS		"/etc/osdchatd.conf"
#define CONFIG_NOPTS		18

#define DEFAULT_XOSD_COLOR	"red"
#define DEFAULT_XOSD_FONT	"-misc-*-*-*-*-*-20-*-*-*-*-*-*-*"

#define MAX_DELAY		60	/* In minutes; does effect infinite (0-zero)*/
#define MAX_WARNLEVEL		200
#define MAX_LINES		16	/* MAX_LINES*MAX_LINECHAR must be */
#define MAX_LINECHAR		200	/*   less than BUFSIZ-1 (4095)    */
#define MIN_LINECHAR		32

#define MAX_NCLIENTS		32	/* overall maximum number of clients */
#define MAX_IPBLOCKS		32	/* maximum number of blocked IPs */

/* user preference of xosd settings structure */
struct xosd_s {
	int offset;		/* pixel offset */
	int delay;		/* seconds before osd hides */
	int lines;		/* max number of lines for xosd to handle */
	int pos;		/* position: top or bottom */
	char *color;		/* osd text color */
	char *font;		/* osd text font */
};

/* user preference structure */
struct pref_s {
	int serv_port;		/* port # to listen to connections */
	int wrapcol;		/* max # of characters per line w/o prompt */
	int move;		/* scroll aging text up or down */
	int max_clients;	/* max # of clients server should handle */
	int max_warnlevel;	/* maximum warning level */
	char *display;		/* X display string */
	char *p_format;		/* prompt format */

	/* bit fields */
	int logmsgs:1;		/* log messages to file */
	int timestamp:1;	/* show timestamps */
	int loud_connect:1;	/* announce client connecting */
	int loud_disconnect:1;	/* announce client disconnecting */
	int loud_block:1;	/* announce blocked client trying to connect */
	int blocklist:1;	/* block clients that exceeded warn level */
};

/* used for converting config file to an arg list */
struct config_set {
	char *opt;		/* config option full name */
	char *arg;		/* switch equivalient of config opt full name */
	int  narg;		/* whether switch requires another argument */
};

/* movement style of aging text: UP or DOWN */
enum {	UP = 0,
	DOWN = 1
};

/* global vars */
xosd	*osd;			/* xosd struct for handling osd stuff */
char 	**osdbuf;		/* pointer array for xosd lines */
fd_set  read_fds;		/* master descriptor set for select() */
int	max_fd;			/* max file descriptor used for select() */
int	num_clients;		/* number of clients connected */

struct passwd *user_info;	/* need user's home dir and login name */
char	pid_fname[128];		/* pid filename */

int	reload_config;		/* set flag upon receiving a SIGHUP */
int 	quit;			/* set flag upon receving various signals */

/* ::function prototypes:: */

/* osdchatd.c */
int	srv_setup();
void	srv_start(int);
void 	srv_add_client(int, struct sockaddr_in *);
void 	srv_disconnect_client(int);
void	srv_add_prompt(int);
void	srv_old_client(int);
void	srv_set_quit(int);
void	srv_quit();
void	srv_sighup(int);
void	srv_reload_config();
void	srv_kill(void);
int	check_warnlevel(int);
void 	log_msgs(char *);
FILE*	safe_fopen(const char *, const char *);

/* osd.c */
int	init_xosd(void);
int	show_osd(char *, char *);

/* config.c */
void	print_usage();
int	find_config_opt(struct config_set *);
void	parse_config();
void	parse_args(int, char **);

/* cs.c */
void	cs_init(void);
char*	cs_get_prompt(int);
int	cs_set_prompt(int, char *);
int	cs_add_client(int, struct sockaddr_in *);
int	cs_del_client(int);
void	cs_del_close_all(void);
int	cs_set_tm(int, time_t);
time_t	cs_get_tm(int);
int	cs_adj_warning(int, int);

/* block.c */
void	cs_block_init(void);
void	cs_block_free(void);
int	cs_block(int);
int	cs_is_blocked(int);

#endif
